@@ -5,7 +5,7 @@
 
 from dataclasses import dataclass, field
 from pathlib import Path
-from typing import List, Optional
+from typing import List, Optional, Set
 
 
 TEXT: str = "text"
@@ -28,7 +28,6 @@ class CommandArguments:
     logging_sections: Optional[str] = None
     log_identifier: Optional[str] = None
     logger: Optional[str] = None
-    formatter: Optional[str] = None
     targets: List[str] = field(default_factory=list)
     use_buck_builder: Optional[bool] = None
     use_buck_source_database: Optional[bool] = None
@@ -50,6 +49,9 @@ class CommandArguments:
     use_command_v2: Optional[bool] = None
     isolation_prefix: Optional[str] = None
     python_version: Optional[str] = None
+    shared_memory_heap_size: Optional[int] = None
+    shared_memory_dependency_table_power: Optional[int] = None
+    shared_memory_hash_table_power: Optional[int] = None
 
 
 @dataclass(frozen=True)
@@ -80,7 +82,48 @@ class IncrementalArguments:
     start_arguments: StartArguments = field(default_factory=StartArguments)
 
 
+@dataclass(frozen=True)
+class CheckArguments:
+    debug: bool = False
+    enable_memory_profiling: bool = False
+    enable_profiling: bool = False
+    log_identifier: Optional[str] = None
+    logging_sections: Optional[str] = None
+    noninteractive: bool = False
+    output: str = TEXT
+    sequential: bool = False
+    show_error_traces: bool = False
+
+
+@dataclass(frozen=True)
+class InferArguments:
+    working_directory: Path
+    annotate_attributes: bool = False
+    annotate_from_existing_stubs: bool = False
+    debug_infer: bool = False
+    quote_annotations: bool = False
+    dequalify: bool = False
+    enable_memory_profiling: bool = False
+    enable_profiling: bool = False
+    interprocedural: bool = False
+    log_identifier: Optional[str] = None
+    logging_sections: Optional[str] = None
+    no_future_annotations: bool = False
+    paths_to_modify: Optional[Set[Path]] = None
+    print_only: bool = False
+    read_stdin: bool = False
+    sequential: bool = False
+
+
 @dataclass(frozen=True)
 class RageArguments:
     output: Optional[Path] = None
     server_log_count: Optional[int] = None
+
+
+@dataclass(frozen=True)
+class StatisticsArguments:
+    filter_paths: List[str] = field(default_factory=list)
+    log_identifier: Optional[str] = None
+    log_results: bool = False
+    print_aggregates: bool = False
