@@ -3,14 +3,21 @@
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
+from . import backend_arguments  # noqa F401
+from . import check  # noqa F401
+from . import coverage  # noqa F401
 from . import incremental  # noqa F401
+from . import infer  # noqa F401
+from . import initialize  # noqa F401
 from . import kill  # noqa F401
 from . import persistent  # noqa F401
+from . import profile  # noqa F401
 from . import pysa_server  # noqa F401
 from . import query  # noqa F401
 from . import rage  # noqa F401
 from . import restart  # noqa F401
 from . import servers  # noqa F401
 from . import start  # noqa F401
+from . import statistics  # noqa F401
 from . import stop  # noqa F401
 from . import validate_models  # noqa F401
