@@ -3,10 +3,10 @@
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
-
 import dataclasses
 import json
 import logging
+import os
 import sys
 from pathlib import Path
 from typing import Any, Dict, Sequence, Union, Optional
@@ -67,6 +67,21 @@ class Error:
             message = f"Cannot parse JSON: {decode_error}"
             raise ErrorParsingFailure(message) from decode_error
 
+    def relativize_path(self, against: Path) -> "Error":
+        relativized_path = Path(os.path.relpath(str(self.path), str(against)))
+        return Error(
+            line=self.line,
+            column=self.column,
+            stop_line=self.stop_line,
+            stop_column=self.stop_column,
+            path=relativized_path,
+            code=self.code,
+            name=self.name,
+            description=self.description,
+            long_description=self.long_description,
+            concise_description=self.concise_description,
+        )
+
     def to_json(self) -> Dict[str, Any]:
         return {
             "line": self.line,
@@ -90,17 +105,14 @@ class Error:
 
 class LegacyError:
     error: Error
-    inference: str
     ignore_error: bool = False
 
     def __init__(
         self,
         error: Error,
-        inference: str,
         ignore_error: bool,
     ) -> None:
         self.error = error
-        self.inference = inference
         self.ignore_error = ignore_error
 
     @staticmethod
@@ -110,14 +122,12 @@ class LegacyError:
     ) -> "LegacyError":
         return LegacyError(
             error=Error.from_json(error),
-            inference=error["inference"],
             ignore_error=ignore_error or error.get("ignore_error", False),
         )
 
     def with_path(self, path: str) -> "LegacyError":
         return LegacyError(
             error=dataclasses.replace(self.error, path=path),
-            inference=self.inference,
             ignore_error=self.ignore_error,
         )
 
@@ -164,7 +174,6 @@ class LegacyError:
 
     def to_json(self) -> Dict[str, Any]:
         error_mapping = self.error.to_json()
-        error_mapping["inference"] = self.inference
         error_mapping["ignore_error"] = self.ignore_error
         return error_mapping
 
