@@ -3,6 +3,7 @@
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
+from __future__ import annotations
 
 import json
 import logging
@@ -13,7 +14,7 @@ import time
 import traceback
 from dataclasses import replace
 from pathlib import Path
-from typing import Iterable, List, Optional
+from typing import Iterable, List
 
 import click
 
@@ -40,7 +41,7 @@ def _log_statistics(
     command: Command,
     start_time: float,
     client_exception_message: str,
-    error_message: Optional[str],
+    error_message: str | None,
     exit_code: int,
 ) -> None:
     configuration = command.configuration
@@ -63,17 +64,13 @@ def _log_statistics(
         )
 
 
-def _show_pyre_version_as_text(
-    binary_version: Optional[str], client_version: str
-) -> None:
+def _show_pyre_version_as_text(binary_version: str | None, client_version: str) -> None:
     if binary_version:
         log.stdout.write(f"Binary version: {binary_version}\n")
     log.stdout.write(f"Client version: {__version__}\n")
 
 
-def _show_pyre_version_as_json(
-    binary_version: Optional[str], client_version: str
-) -> None:
+def _show_pyre_version_as_json(binary_version: str | None, client_version: str) -> None:
     version_json = {
         **({} if binary_version is None else {"binary": binary_version}),
         "client": client_version,
@@ -82,7 +79,7 @@ def _show_pyre_version_as_json(
 
 
 def _show_pyre_version(arguments: command_arguments.CommandArguments) -> None:
-    binary_version: Optional[str] = None
+    binary_version: str | None = None
     client_version: str = __version__
     try:
         configuration = configuration_module.create_configuration(arguments, Path("."))
@@ -95,6 +92,12 @@ def _show_pyre_version(arguments: command_arguments.CommandArguments) -> None:
         _show_pyre_version_as_text(binary_version, client_version)
 
 
+def _start_logging_to_directory(log_directory: str) -> None:
+    log_directory_path = Path(log_directory)
+    log_directory_path.mkdir(parents=True, exist_ok=True)
+    log.enable_file_logging(log_directory_path / "pyre.stderr")
+
+
 def run_pyre_command(
     command: Command,
     configuration: configuration_module.Configuration,
@@ -108,7 +111,7 @@ def run_pyre_command(
     exit_code = ExitCode.FAILURE
     try:
         _check_configuration(configuration)
-        log.start_logging_to_directory(noninteractive, configuration.log_directory)
+        _start_logging_to_directory(configuration.log_directory)
         LOG.debug(f"Running cli command `{' '.join(sys.argv)}`...")
         exit_code = command.run().exit_code()
     except (buck.BuckException, EnvironmentException) as error:
@@ -123,7 +126,7 @@ def run_pyre_command(
         exit_code = ExitCode.CONFIGURATION_ERROR
     except commands.ClientException as error:
         client_exception_message = str(error)
-        exit_code = ExitCode.FAILURE
+        exit_code = error.exit_code
     except Exception:
         client_exception_message = traceback.format_exc()
         exit_code = ExitCode.FAILURE
@@ -149,13 +152,29 @@ def run_pyre_command(
 
 def _run_check_command(arguments: command_arguments.CommandArguments) -> ExitCode:
     configuration = _create_configuration_with_retry(arguments, Path("."))
-    return run_pyre_command(
-        commands.Check(
-            arguments, original_directory=os.getcwd(), configuration=configuration
-        ),
-        configuration,
-        arguments.noninteractive,
-    )
+    if configuration.use_command_v2:
+        _check_configuration(configuration)
+        _start_logging_to_directory(configuration.log_directory)
+        check_arguments = command_arguments.CheckArguments(
+            debug=arguments.debug,
+            enable_memory_profiling=arguments.enable_memory_profiling,
+            enable_profiling=arguments.enable_profiling,
+            log_identifier=arguments.log_identifier,
+            logging_sections=arguments.logging_sections,
+            noninteractive=arguments.noninteractive,
+            output=arguments.output,
+            sequential=arguments.sequential,
+            show_error_traces=arguments.show_error_traces,
+        )
+        return v2.check.run(configuration, check_arguments)
+    else:
+        return run_pyre_command(
+            commands.Check(
+                arguments, original_directory=os.getcwd(), configuration=configuration
+            ),
+            configuration,
+            arguments.noninteractive,
+        )
 
 
 def _run_incremental_command(
@@ -168,9 +187,7 @@ def _run_incremental_command(
     configuration = _create_configuration_with_retry(arguments, Path("."))
     if configuration.use_command_v2:
         _check_configuration(configuration)
-        log.start_logging_to_directory(
-            arguments.noninteractive, configuration.log_directory
-        )
+        _start_logging_to_directory(configuration.log_directory)
         start_arguments = command_arguments.StartArguments(
             changed_files_path=arguments.changed_files_path,
             debug=arguments.debug,
@@ -237,8 +254,8 @@ def _run_default_command(arguments: command_arguments.CommandArguments) -> ExitC
 def _run_servers_list_command(
     arguments: command_arguments.CommandArguments,
 ) -> ExitCode:
-    if arguments.use_command_v2:
-        return v2.servers.run_list()
+    if arguments.use_command_v2 is not False:
+        return v2.servers.run_list(arguments.output)
     else:
         configuration = configuration_module.create_configuration(arguments, Path("."))
         return run_pyre_command(
@@ -362,7 +379,6 @@ def _check_configuration(configuration: configuration_module.Configuration) -> N
 @click.option("--log-identifier", type=str, default=None, hidden=True)
 @click.option("--dot-pyre-directory", type=str, hidden=True)
 @click.option("--logger", type=str, hidden=True)
-@click.option("--formatter", type=str, hidden=True)
 @click.option(
     "--target",
     type=str,
@@ -429,7 +445,9 @@ def _check_configuration(configuration: configuration_module.Configuration) -> N
 @click.option("--changed-files-path", type=str, hidden=True)
 @click.option("--saved-state-project", type=str, hidden=True)
 @click.option("--features", type=str, hidden=True)
-@click.option("--use-command-v2", is_flag=True, default=None, hidden=True)
+@click.option(
+    "--use-command-v2/--no-use-command-v2", is_flag=True, default=None, hidden=True
+)
 @click.option("--isolation-prefix", type=str, hidden=True)
 @click.option(
     "--python-version",
@@ -441,44 +459,64 @@ def _check_configuration(configuration: configuration_module.Configuration) -> N
     ),
     hidden=True,
 )
+@click.option(
+    "--shared-memory-heap-size",
+    type=int,
+    help="Size of the shared memory heap, in bytes.",
+    hidden=True,
+)
+@click.option(
+    "--shared-memory-dependency-table-power",
+    type=int,
+    help="Power of the dependency table in shared memory.",
+    hidden=True,
+)
+@click.option(
+    "--shared-memory-hash-table-power",
+    type=int,
+    help="Power of the hash table in shared memory.",
+    hidden=True,
+)
 def pyre(
     context: click.Context,
-    local_configuration: Optional[str],
+    local_configuration: str | None,
     version: bool,
     debug: bool,
-    sequential: Optional[bool],
-    strict: Optional[bool],
+    sequential: bool | None,
+    strict: bool | None,
     additional_check: Iterable[str],
     show_error_traces: bool,
     output: str,
     enable_profiling: bool,
     enable_memory_profiling: bool,
     noninteractive: bool,
-    logging_sections: Optional[str],
-    log_identifier: Optional[str],
-    dot_pyre_directory: Optional[str],
-    logger: Optional[str],
-    formatter: Optional[str],
+    logging_sections: str | None,
+    log_identifier: str | None,
+    dot_pyre_directory: str | None,
+    logger: str | None,
     target: Iterable[str],
-    use_buck_builder: Optional[bool],
-    buck_mode: Optional[str],
-    use_buck_source_database: Optional[bool],
+    use_buck_builder: bool | None,
+    buck_mode: str | None,
+    use_buck_source_database: bool | None,
     source_directory: Iterable[str],
-    filter_directory: Optional[str],
+    filter_directory: str | None,
     no_saved_state: bool,
     search_path: Iterable[str],
-    binary: Optional[str],
-    buck_builder_binary: Optional[str],
+    binary: str | None,
+    buck_builder_binary: str | None,
     exclude: Iterable[str],
-    typeshed: Optional[str],
-    save_initial_state_to: Optional[str],
-    load_initial_state_from: Optional[str],
-    changed_files_path: Optional[str],
-    saved_state_project: Optional[str],
-    features: Optional[str],
-    use_command_v2: Optional[bool],
-    isolation_prefix: Optional[str],
-    python_version: Optional[str],
+    typeshed: str | None,
+    save_initial_state_to: str | None,
+    load_initial_state_from: str | None,
+    changed_files_path: str | None,
+    saved_state_project: str | None,
+    features: str | None,
+    use_command_v2: bool | None,
+    isolation_prefix: str | None,
+    python_version: str | None,
+    shared_memory_heap_size: int | None,
+    shared_memory_dependency_table_power: int | None,
+    shared_memory_hash_table_power: int | None,
 ) -> int:
     arguments = command_arguments.CommandArguments(
         local_configuration=local_configuration,
@@ -495,7 +533,6 @@ def pyre(
         logging_sections=logging_sections,
         log_identifier=log_identifier,
         logger=logger,
-        formatter=formatter,
         targets=list(target),
         use_buck_builder=use_buck_builder,
         use_buck_source_database=use_buck_source_database,
@@ -519,6 +556,9 @@ def pyre(
         use_command_v2=use_command_v2,
         isolation_prefix=isolation_prefix,
         python_version=python_version,
+        shared_memory_heap_size=shared_memory_heap_size,
+        shared_memory_dependency_table_power=shared_memory_dependency_table_power,
+        shared_memory_hash_table_power=shared_memory_hash_table_power,
     )
     if arguments.version:
         _show_pyre_version(arguments)
@@ -554,6 +594,8 @@ def pyre(
     help="Directory to write analysis results to.",
 )
 @click.option("--dump-call-graph", is_flag=True, default=False, hidden=True)
+# pyre-fixme[56]: Pyre was not able to infer the type of argument `os.path.abspath`
+#  to decorator factory `click.option`.
 @click.option("--repository-root", type=os.path.abspath)
 @click.option(
     "--rule",
@@ -589,21 +631,27 @@ def pyre(
     type=int,
     help="Limit the trace length of taint flows.",
 )
+@click.option(
+    "--maximum-tito-depth",
+    type=int,
+    help="Limit the depth of inferred taint-in-taint-out in taint flows.",
+)
 @click.pass_context
 def analyze(
     context: click.Context,
     analysis: str,
     taint_models_path: Iterable[str],
     no_verify: bool,
-    save_results_to: Optional[str],
+    save_results_to: str | None,
     dump_call_graph: bool,
-    repository_root: Optional[str],
+    repository_root: str | None,
     rule: Iterable[int],
-    find_missing_flows: Optional[str],
+    find_missing_flows: str | None,
     dump_model_query_results: bool,
     use_cache: bool,
     inline_decorators: bool,
-    maximum_trace_length: Optional[int],
+    maximum_trace_length: int | None,
+    maximum_tito_depth: int | None,
 ) -> int:
     """
     Run Pysa, the inter-procedural static analysis tool.
@@ -632,6 +680,7 @@ def analyze(
             use_cache=use_cache,
             inline_decorators=inline_decorators,
             maximum_trace_length=maximum_trace_length,
+            maximum_tito_depth=maximum_tito_depth,
         ),
         configuration,
         command_argument.noninteractive,
@@ -701,31 +750,21 @@ def incremental(
 
 
 @pyre.command()
-@click.argument("modify_paths", type=filesystem.file_or_directory_exists, nargs=-1)
+@click.argument(
+    "paths_to_modify",
+    type=filesystem.file_or_directory_exists,
+    nargs=-1,
+)
 @click.option(
     "-p",
     "--print-only",
     is_flag=True,
     default=False,
     help=(
-        "Print raw JSON errors to standard output, without converting to stubs "
-        "or annnotating."
+        "Print raw JSON inference data to standard output, without "
+        "converting to stubs or annnotating."
     ),
 )
-@click.option(
-    "-f",
-    "--full-only",
-    is_flag=True,
-    default=False,
-    help="Only output fully annotated functions.",
-)
-@click.option(
-    "-r",
-    "--recursive",
-    is_flag=True,
-    default=False,
-    help="Recursively run infer until no new annotations are generated.",
-)
 @click.option(
     "-i",
     "--in-place",
@@ -733,12 +772,6 @@ def incremental(
     default=False,
     help="Modifies original files and add inferred annotations to all functions.",
 )
-@click.option(
-    "--json",
-    is_flag=True,
-    default=False,
-    help="Accept JSON input instead of running full check.",
-)
 @click.option(
     "--annotate-from-existing-stubs",
     is_flag=True,
@@ -752,47 +785,120 @@ def incremental(
     help="Print error message when file fails to annotate.",
 )
 @click.option(
-    "--full-stubs",
+    "--read-stdin",
     is_flag=True,
     default=False,
-    help="Include pre-existing annotations in the generated stubs.",
+    help="Read input from stdin instead of running a full infer.",
+)
+@click.option(
+    "--annotate-attributes",
+    is_flag=True,
+    default=False,
+    help=(
+        "Allow infer to attempt to annotate class attributes? "
+        "The code-generation logic for this is incomplete, so the "
+        "default is False but you may manually enable it."
+    ),
+)
+@click.option(
+    "--no-future-annotations",
+    is_flag=True,
+    default=False,
+    help=(
+        "Infer assumes it can insert unquoted annotations anywhere in a "
+        "module. In python versions between 3.7 and 3.10, we can ensure this "
+        "is valid by inserting `from __future__ import annotations`, which"
+        "we do by default. You can use this flag to disable the import, which "
+        "is illegal in pre-3.7 python versions and is unnecessary (but allowed) "
+        "in python 3.10+. This flag disables inserting the import."
+    ),
+)
+@click.option(
+    "--quote-annotations",
+    is_flag=True,
+    default=False,
+    help=(
+        "Quote all added type annotations? "
+        "This is recommended when using pyre infer prior to pysa "
+        "because it allows us to avoid introducing imports, which "
+        "is important because then the line numbers match checked-in "
+        "code."
+    ),
+)
+@click.option(
+    "--dequalify",
+    is_flag=True,
+    default=False,
+    help=(
+        "Dequalify all annotations? This is a temporary flag, used to "
+        "force fully-qualified names (e.g. sqlalchemy.sql.schema.Column) "
+        "to be dequalified (e.g. Column). It is needed now because pyre "
+        "infer doesn't yet know how to handle imports and qualified names "
+        "in a principled way."
+    ),
+)
+@click.option(
+    "--interprocedural",
+    is_flag=True,
+    default=False,
+    help=(
+        "Use (experimental) interprocedural inference. "
+        "Not recommended except for pyre developers, this work "
+        "is incomplete."
+    ),
 )
 @click.pass_context
 def infer(
     context: click.Context,
-    modify_paths: Iterable[str],
+    paths_to_modify: Iterable[str],
     print_only: bool,
-    full_only: bool,
-    recursive: bool,
     in_place: bool,
-    json: bool,
     annotate_from_existing_stubs: bool,
     debug_infer: bool,
-    full_stubs: bool,
+    read_stdin: bool,
+    annotate_attributes: bool,
+    no_future_annotations: bool,
+    quote_annotations: bool,
+    dequalify: bool,
+    interprocedural: bool,
 ) -> int:
     """
-    Try adding type annotations to untyped codebase.
+    Run pyre infer.
+
+    The optional PATHS_TO_MODIFY argument is a list of directory or file
+    paths to include when annotating in-place.
+
+    If empty, then we'll annotate all relevant modules in-place, and it is
+    ignored unless the `--in-place` flag is set.
     """
-    in_place_paths = list(modify_paths) if in_place else None
-    full_stub_paths = list(modify_paths) if full_stubs else None
     command_argument: command_arguments.CommandArguments = context.obj["arguments"]
     configuration = _create_configuration_with_retry(command_argument, Path("."))
-    return run_pyre_command(
-        commands.Infer(
-            command_argument,
-            original_directory=os.getcwd(),
-            configuration=configuration,
-            print_errors=print_only,
-            full_only=full_only,
-            recursive=recursive,
-            in_place=in_place_paths,
-            errors_from_stdin=json,
-            annotate_from_existing_stubs=annotate_from_existing_stubs,
-            debug_infer=debug_infer,
-            full_stub_paths=full_stub_paths,
-        ),
+    working_directory = Path.cwd()
+    modify_paths = (
+        None
+        if not in_place
+        else {working_directory / Path(path) for path in paths_to_modify}
+    )
+    return v2.infer.run(
         configuration,
-        command_argument.noninteractive,
+        command_arguments.InferArguments(
+            working_directory=working_directory,
+            annotate_attributes=annotate_attributes,
+            annotate_from_existing_stubs=annotate_from_existing_stubs,
+            enable_memory_profiling=command_argument.enable_memory_profiling,
+            enable_profiling=command_argument.enable_profiling,
+            debug_infer=debug_infer,
+            quote_annotations=quote_annotations,
+            dequalify=dequalify,
+            interprocedural=interprocedural,
+            log_identifier=command_argument.log_identifier,
+            logging_sections=command_argument.logging_sections,
+            no_future_annotations=no_future_annotations,
+            paths_to_modify=modify_paths,
+            print_only=print_only,
+            read_stdin=read_stdin,
+            sequential=command_argument.sequential,
+        ),
     )
 
 
@@ -808,7 +914,7 @@ def init(context: click.Context, local: bool) -> int:
     """
     Create a pyre configuration file at the current directory.
     """
-    return commands.Initialize().run().exit_code()
+    return v2.initialize.run()
 
 
 @pyre.command()
@@ -849,35 +955,21 @@ def persistent(context: click.Context, no_watchman: bool) -> int:
     and responses from the Pyre server to stdout.
     """
     command_argument: command_arguments.CommandArguments = context.obj["arguments"]
+    base_directory = Path(".")
     configuration = configuration_module.create_configuration(
-        command_argument, Path(".")
+        command_argument, base_directory
     )
     if configuration.use_command_v2:
-        log.start_logging_to_directory(
-            # Always log to file regardless of whether `-n` is given
-            noninteractive=False,
-            log_directory=configuration.log_directory,
+        _start_logging_to_directory(
+            configuration.log_directory,
         )
+        # NOTE(grievejia): `--no-watchman` has no effect here. We might want to
+        # either respect it or deprecate it at some point.
         return v2.persistent.run(
-            configuration,
-            command_arguments.StartArguments(
-                changed_files_path=command_argument.changed_files_path,
-                debug=command_argument.debug,
-                enable_memory_profiling=command_argument.enable_memory_profiling,
-                enable_profiling=command_argument.enable_profiling,
-                load_initial_state_from=command_argument.load_initial_state_from,
-                log_identifier=command_argument.log_identifier,
-                logging_sections=command_argument.logging_sections,
-                no_saved_state=command_argument.no_saved_state,
-                no_watchman=no_watchman,
-                noninteractive=command_argument.noninteractive,
-                save_initial_state_to=command_argument.save_initial_state_to,
-                saved_state_project=command_argument.saved_state_project,
-                sequential=command_argument.sequential,
-                show_error_traces=command_argument.show_error_traces,
-                store_type_check_resolution=False,
-                terminal=False,
-                wait_on_initialization=True,
+            command_argument,
+            base_directory,
+            v2.backend_arguments.RemoteLogging.create(
+                configuration.logger, command_argument.log_identifier
             ),
         )
     else:
@@ -906,10 +998,8 @@ def pysa_language_server(context: click.Context, no_watchman: bool) -> int:
     configuration = configuration_module.create_configuration(
         command_argument, Path(".")
     )
-    log.start_logging_to_directory(
-        # Always log to file regardless of whether `-n` is given
-        noninteractive=False,
-        log_directory=configuration.log_directory,
+    _start_logging_to_directory(
+        configuration.log_directory,
     )
     return v2.pysa_server.run(
         configuration,
@@ -956,16 +1046,19 @@ def profile(context: click.Context, profile_output: str) -> int:
 
     command_argument: command_arguments.CommandArguments = context.obj["arguments"]
     configuration = _create_configuration_with_retry(command_argument, Path("."))
-    return run_pyre_command(
-        commands.Profile(
-            command_argument,
-            original_directory=os.getcwd(),
-            configuration=configuration,
-            profile_output=get_profile_output(profile_output),
-        ),
-        configuration,
-        command_argument.noninteractive,
-    )
+    if configuration.use_command_v2:
+        return v2.profile.run(configuration, get_profile_output(profile_output))
+    else:
+        return run_pyre_command(
+            commands.Profile(
+                command_argument,
+                original_directory=os.getcwd(),
+                configuration=configuration,
+                profile_output=get_profile_output(profile_output),
+            ),
+            configuration,
+            command_argument.noninteractive,
+        )
 
 
 @pyre.command()
@@ -984,9 +1077,7 @@ def query(context: click.Context, query: str) -> int:
     configuration = _create_configuration_with_retry(command_argument, Path("."))
 
     if configuration.use_command_v2:
-        log.start_logging_to_directory(
-            command_argument.noninteractive, configuration.log_directory
-        )
+        _start_logging_to_directory(configuration.log_directory)
         return v2.query.run(configuration, query)
     else:
         return run_pyre_command(
@@ -1002,6 +1093,8 @@ def query(context: click.Context, query: str) -> int:
 
 
 @pyre.command()
+# pyre-fixme[56]: Pyre was not able to infer the type of argument `os.path.abspath`
+#  to decorator factory `click.option`.
 @click.option(
     "--output-file",
     type=os.path.abspath,
@@ -1014,9 +1107,7 @@ def query(context: click.Context, query: str) -> int:
     help="Number of server logs to include in the diagnositics. Default to 3.",
 )
 @click.pass_context
-def rage(
-    context: click.Context, output_file: Optional[str], server_log_count: int
-) -> int:
+def rage(context: click.Context, output_file: str | None, server_log_count: int) -> int:
     """
     Collects troubleshooting diagnostics for Pyre, and writes this information
     to the terminal or to a file.
@@ -1089,9 +1180,7 @@ def restart(
     configuration = _create_configuration_with_retry(command_argument, Path("."))
     if configuration.use_command_v2:
         _check_configuration(configuration)
-        log.start_logging_to_directory(
-            command_argument.noninteractive, configuration.log_directory
-        )
+        _start_logging_to_directory(configuration.log_directory)
         start_arguments = command_arguments.StartArguments(
             changed_files_path=command_argument.changed_files_path,
             debug=command_argument.debug,
@@ -1167,7 +1256,7 @@ def servers_stop(context: click.Context) -> int:
     Stop all running servers.
     """
     command_argument: command_arguments.CommandArguments = context.obj["arguments"]
-    if command_argument.use_command_v2:
+    if command_argument.use_command_v2 is not False:
         return v2.servers.run_stop()
     else:
         configuration = configuration_module.create_configuration(
@@ -1234,9 +1323,7 @@ def start(
     configuration = _create_configuration_with_retry(command_argument, Path("."))
     if configuration.use_command_v2:
         _check_configuration(configuration)
-        log.start_logging_to_directory(
-            command_argument.noninteractive, configuration.log_directory
-        )
+        _start_logging_to_directory(configuration.log_directory)
         return v2.start.run(
             configuration,
             command_arguments.StartArguments(
@@ -1278,34 +1365,89 @@ def start(
 
 
 @pyre.command()
-# TODO[T60916205]: Rename this argument, it doesn't make sense anymore
-@click.argument("filter_paths", type=filesystem.exists, nargs=-1)
+@click.argument("filter_paths", type=str, nargs=-1)
 @click.option(
     "--log-results",
     is_flag=True,
     default=False,
     help="Log the statistics results to external tables.",
 )
+@click.option(
+    "--print-aggregates",
+    is_flag=True,
+    default=False,
+    help="Print aggregate instead of per-path data.",
+)
 @click.pass_context
 def statistics(
-    context: click.Context, filter_paths: Iterable[str], log_results: bool
+    context: click.Context,
+    filter_paths: Iterable[str],
+    log_results: bool,
+    print_aggregates: bool,
 ) -> int:
     """
     Collect various syntactic metrics on type coverage.
     """
     command_argument: command_arguments.CommandArguments = context.obj["arguments"]
     configuration = _create_configuration_with_retry(command_argument, Path("."))
-    return run_pyre_command(
-        commands.Statistics(
-            command_argument,
-            original_directory=os.getcwd(),
-            configuration=configuration,
-            filter_paths=list(filter_paths),
-            log_results=log_results,
-        ),
-        configuration,
-        command_argument.noninteractive,
-    )
+
+    if configuration.use_command_v2:
+        return v2.statistics.run(
+            configuration,
+            command_arguments.StatisticsArguments(
+                filter_paths=list(filter_paths),
+                log_identifier=command_argument.log_identifier,
+                log_results=log_results,
+                print_aggregates=print_aggregates,
+            ),
+        )
+    else:
+        return run_pyre_command(
+            commands.Statistics(
+                command_argument,
+                original_directory=os.getcwd(),
+                configuration=configuration,
+                filter_paths=list(filter_paths),
+                log_results=log_results,
+                aggregate=print_aggregates,
+            ),
+            configuration,
+            command_argument.noninteractive,
+        )
+
+
+@pyre.command()
+@click.option(
+    "--working-directory",
+    metavar="DIR",
+    default=os.curdir,
+    show_default="current directory",
+    type=str,
+    help="In the output, make paths relative to directory specified.",
+)
+@click.pass_context
+def coverage(
+    context: click.Context,
+    working_directory: str,
+) -> int:
+    """
+    Collect line-level type coverage.
+    """
+    command_argument: command_arguments.CommandArguments = context.obj["arguments"]
+    configuration = _create_configuration_with_retry(command_argument, Path("."))
+    if configuration.use_command_v2:
+        return v2.coverage.run(configuration, working_directory)
+    else:
+        return run_pyre_command(
+            commands.Coverage(
+                command_argument,
+                original_directory=os.getcwd(),
+                configuration=configuration,
+                working_directory=working_directory,
+            ),
+            configuration,
+            command_argument.noninteractive,
+        )
 
 
 @pyre.command()
@@ -1319,9 +1461,7 @@ def stop(context: click.Context) -> int:
         command_argument, Path(".")
     )
     if configuration.use_command_v2:
-        log.start_logging_to_directory(
-            command_argument.noninteractive, configuration.log_directory
-        )
+        _start_logging_to_directory(configuration.log_directory)
         return v2.stop.run(configuration)
     else:
         return run_pyre_command(
@@ -1345,9 +1485,7 @@ def validate_models(context: click.Context) -> int:
     configuration = _create_configuration_with_retry(command_argument, Path("."))
 
     if configuration.use_command_v2:
-        log.start_logging_to_directory(
-            command_argument.noninteractive, configuration.log_directory
-        )
+        _start_logging_to_directory(configuration.log_directory)
         return v2.validate_models.run(configuration, output=command_argument.output)
     else:
         return run_pyre_command(
@@ -1373,6 +1511,9 @@ def main(argv: List[str] = sys.argv[1:]) -> int:
         except click.ClickException as error:
             error.show()
             return_code = ExitCode.FAILURE
+        except commands.ClientException as error:
+            LOG.error(str(error))
+            return_code = error.exit_code
         except Exception as error:
             LOG.error(str(error))
             return_code = ExitCode.FAILURE
