@@ -42,11 +42,11 @@ class Configuration:
             "source_directories"
         )
         self.version: Optional[str] = json_contents.get("version")
-        self.differential: bool = json_contents.get("differential", False)
         self.use_buck_builder: Optional[bool] = json_contents.get("use_buck_builder")
         self.use_buck_source_database: Optional[bool] = json_contents.get(
             "use_buck_source_database"
         )
+        self.use_command_v2: Optional[bool] = json_contents.get("use_command_v2")
 
     def get_contents(self) -> Dict[str, Any]:
         """Assumption: The field names in this class match the key names in
@@ -66,6 +66,7 @@ class Configuration:
         update_contents("strict")
         update_contents("use_buck_builder")
         update_contents("use_buck_source_database")
+        update_contents("use_command_v2")
         return contents
 
     @staticmethod
@@ -155,6 +156,9 @@ class Configuration:
         self.use_buck_builder = True
         self.use_buck_source_database = True
 
+    def enable_new_server(self) -> None:
+        self.use_command_v2 = True
+
     def add_strict(self) -> None:
         if self.strict:
             LOG.info("Configuration is already strict.")
