@@ -13,6 +13,7 @@ from logging import Logger
 from . import UserError
 from .ast import UnstableAST
 from .commands.codemods import (
+    EnableNewServer,
     EnableSourceDatabaseBuckBuilder,
     MissingGlobalAnnotations,
     MissingOverrideReturnAnnotations,
@@ -23,7 +24,6 @@ from .commands.fix_configuration import FixConfiguration
 from .commands.fixme import Fixme
 from .commands.fixme_all import FixmeAll
 from .commands.fixme_single import FixmeSingle
-from .commands.fixme_targets import FixmeTargets
 from .commands.global_version_update import GlobalVersionUpdate
 from .commands.strict_default import StrictDefault
 from .commands.support_sqlalchemy import SupportSqlalchemy
@@ -82,10 +82,6 @@ def run(repository: Repository) -> None:
     fixme_all = commands.add_parser("fixme-all")
     FixmeAll.add_arguments(fixme_all)
 
-    # Subcommand: Fixme all errors in targets running type checking
-    fixme_targets = commands.add_parser("fixme-targets")
-    FixmeTargets.add_arguments(fixme_targets)
-
     # Subcommand: Remove targets integration and replace with configuration
     targets_to_configuration = commands.add_parser("targets-to-configuration")
     TargetsToConfiguration.add_arguments(targets_to_configuration)
@@ -110,6 +106,9 @@ def run(repository: Repository) -> None:
     support_sqlalchemy = commands.add_parser("support-sqlalchemy")
     SupportSqlalchemy.add_arguments(support_sqlalchemy)
 
+    enable_new_server = commands.add_parser("enable-new-server")
+    EnableNewServer.add_arguments(enable_new_server)
+
     # Initialize default values.
     arguments = parser.parse_args()
     if not hasattr(arguments, "command"):
